@@ -2,9 +2,9 @@
 ####**********************************************************************
 ####
 ####  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-####  Version 1.6.1
+####  Version 2.0.0 (_PROJECT_BUILD_ID_)
 ####
-####  Copyright 2012, University of Miami
+####  Copyright 2015, University of Miami
 ####
 ####  This program is free software; you can redistribute it and/or
 ####  modify it under the terms of the GNU General Public License
@@ -52,7 +52,7 @@
 ####    5425 Nestleway Drive, Suite L1
 ####    Clemmons, NC 27012
 ####
-####    email:  commerce@kogalur.com
+####    email:  ubk@kogalur.com
 ####    URL:    http://www.kogalur.com
 ####    --------------------------------------------------------------
 ####
@@ -61,75 +61,81 @@
 
 
 find.interaction.rfsrc <- function(
-    object, 
-    xvar.names,
-    cause,
-    importance = c("permute", "random", "permute.ensemble", "random.ensemble"),
-    method = c("maxsubtree", "vimp"),
-    sorted = TRUE,
-    nvar = NULL, 
-    nrep  = 1,
-    subset,
-    na.action = c("na.omit", "na.impute", "na.random"),
-    seed = NULL,
-    do.trace = FALSE,
-    verbose = TRUE,
-    ...)
+  object, 
+  xvar.names,
+  cause,
+  outcome.target = NULL,
+  importance = c("permute", "random", "anti", "permute.ensemble", "random.ensemble", "anti.ensemble"),
+  method = c("maxsubtree", "vimp"),
+  sorted = TRUE,
+  nvar = NULL, 
+  nrep  = 1,
+  subset,
+  na.action = c("na.omit", "na.impute", "na.random"),
+  seed = NULL,
+  do.trace = FALSE,
+  verbose = TRUE,
+  ...)
 {
   if (is.null(object)) stop("Object is empty!")
   if (sum(inherits(object, c("rfsrc", "grow"), TRUE) == c(1, 2)) != 2    &
       sum(inherits(object, c("rfsrc", "forest"), TRUE) == c(1, 2)) != 2)
     stop("This function only works for objects of class `(rfsrc, grow)' or '(rfsrc, forest)'.")
-    if (sum(inherits(object, c("rfsrc", "grow"), TRUE) == c(1, 2)) == 2) {
-      if (is.null(object$forest)) 
-        stop("Forest is empty!  Re-run grow call with forest set to 'TRUE'.")
-    }
+  if (sum(inherits(object, c("rfsrc", "grow"), TRUE) == c(1, 2)) == 2) {
+    if (is.null(object$forest)) 
+      stop("Forest is empty!  Re-run grow call with forest set to 'TRUE'.")
+  }
   if (missing(cause)) {
     cause <- 1
   }
-  outcome.target <- get.outcome.target(object$family, outcome.target)
   if (object$family == "unsupv") {
     method <- "maxsubtree"
   }
   method <- match.arg(method,  c("maxsubtree", "vimp"))
-  importance <- match.arg(importance, c("permute", "random", "permute.ensemble", "random.ensemble"))
+  importance <- match.arg(importance, c("permute", "random", "anti", "permute.ensemble", "random.ensemble", "anti.ensemble"))
   event.info <- get.event.info(object)
   n.event <- max(1, length(event.info$event.type))
+  outcome.target <- coerce.multivariate.target(object, outcome.target)
   if (n.event > 1) {
     object.imp <- NULL
+    interact.imp.list.names <- paste("event.", 1:length(event.info$event.type), sep = "")
   }
-  else {
-    object.imp <- cbind(object$importance)[, cause, drop = FALSE]
-  }
-  cov.names <- object$xvar.names
-  n.interact <- n.cov <- length(cov.names)
-  if (!missing(xvar.names)) {
-    if (sum(is.element(cov.names, xvar.names)) == 0) {
-      stop("Variables do not match original analysis:", cov.names)
+    else {
+      object.imp <- cbind(coerce.multivariate(object, outcome.target)$importance)[, cause, drop = FALSE]
     }
-    xvar.names <- unique(xvar.names[is.element(xvar.names, cov.names)])
-    if (length(xvar.names) == 1)
+  xvar.org.names <- object$xvar.names
+  if (!missing(xvar.names)) {
+    if (sum(is.element(xvar.org.names, xvar.names)) == 0) {
+      stop("Variables do not match original analysis:", xvar.names)
+    }
+    xvar.names <- unique(xvar.names[is.element(xvar.names, xvar.org.names)])
+    if (length(xvar.names) == 1) 
       stop("Pairwise comparisons require more than one candidate variable.")
-    cov.names <- xvar.names
-    sorted <- FALSE
-    n.interact <- length(xvar.names)
   }
+    else {
+      xvar.names <- xvar.org.names
+    }
+  n.interact <- length(xvar.names)
   if (sorted) {
     if (!is.null(object.imp)) {
-      o.r <- order(object.imp, decreasing = TRUE)
-      cov.names <- cov.names[o.r]
+      o.r <- order(object.imp[xvar.names, ], decreasing = TRUE)
+      xvar.names <- xvar.names[o.r]
     }
   }
   if (!missing(nvar)) {
-    n.interact <- min(n.cov, max(round(nvar), 1))
+    n.interact <- min(n.interact, max(round(nvar), 1))
+    xvar.names <- xvar.names[1:n.interact]
   }
   if (n.interact == 1) {
     stop("Pairwise comparisons require more than one candidate variable.")
   }
   if (method == "vimp") {
+    outcome.target <- coerce.multivariate.target(object, outcome.target)
+    yvar.dim <- ncol(object$yvar)
+    family.org <- object$family
     if (n.event > 1) {
       interact.imp.list <- vector("list", n.event)
-      names(interact.imp.list) <- colnames(object$importance)
+      names(interact.imp.list) <- interact.imp.list.names
     }
     for (j in 1:n.event) {
       rownames.interact.imp <- interact.imp <- NULL
@@ -138,34 +144,35 @@ find.interaction.rfsrc <- function(
         cat("--> event", j, "\n")
       }
       for (k in 1:(n.interact-1)) {
-        n.joint.cov <- n.interact - k
-        imp <- rep(0 , 1 + n.joint.cov)
-        imp.joint <- rep(0, n.joint.cov)
+        n.joint.var <- n.interact - k
+        imp <- rep(0 , 1 + n.joint.var)
+        imp.joint <- rep(0, n.joint.var)
         for (l in (k+1):n.interact) {
           if (verbose) {
-            cat("Pairing",cov.names[k],"with",cov.names[l],"\n")
+            cat("Pairing",xvar.names[k],"with",xvar.names[l],"\n")
           }
           for (m in 1:nrep) {
-            imp.indv.m <- c(cbind(vimp(object, cov.names[c(k,l)], outcome.target = outcome.target,
-                                       importance = importance, joint = FALSE, 
-                                       na.action = na.action, subset = subset, seed = seed, 
-                                       do.trace = do.trace)$importance)[, target.dim])
-            imp.joint.m <- vimp(object, cov.names[c(k,l)], outcome.target = outcome.target,
-                                importance = importance, joint = TRUE,
-                                na.action = na.action, subset = subset, seed = seed,
-                                do.trace = do.trace)$importance[target.dim]
+            imp.indv.m <- c(cbind(coerce.multivariate(vimp(object, xvar.names[c(k,l)],
+                                                           outcome.target = outcome.target,
+                                                           importance = importance, joint = FALSE, 
+                                                           na.action = na.action, subset = subset, seed = seed, 
+                                                           do.trace = do.trace), outcome.target)$importance)[, target.dim])
+            imp.joint.m <- coerce.multivariate(vimp(object, xvar.names[c(k,l)], outcome.target = outcome.target,
+                                                    importance = importance, joint = TRUE,
+                                                    na.action = na.action, subset = subset, seed = seed,
+                                                    do.trace = do.trace), outcome.target)$importance[target.dim]
             imp[1] <- imp[1] + imp.indv.m[1]
             imp[l-k+1] <- imp[l-k+1] + imp.indv.m[2]
             imp.joint[l-k] <- imp.joint[l-k] + imp.joint.m
           }
         }
-        imp[1] <- imp[1] / n.joint.cov
+        imp[1] <- imp[1] / n.joint.var
         imp <- imp/nrep
         imp.joint <- imp.joint/nrep
         interact.imp <- rbind(interact.imp,
                               cbind(imp[1], imp[-1], imp.joint, (imp[1] + imp)[-1], imp.joint - (imp[1] + imp)[-1]))
         rownames.interact.imp <- c(rownames.interact.imp,
-                                   paste(cov.names[k],":",cov.names[(k+1):n.interact],
+                                   paste(xvar.names[k],":",xvar.names[(k+1):n.interact],
                                          sep=""))
       }
       colnames(interact.imp) <- c("Var 1", "Var 2","Paired","Additive","Difference")
@@ -180,7 +187,11 @@ find.interaction.rfsrc <- function(
     if (verbose) {
       cat("\n")
       cat("                              Method: ", method,                       "\n", sep="")
-      cat("                    No. of variables: ", n.cov,                        "\n", sep="")
+      cat("                    No. of variables: ", n.interact,                   "\n", sep="")
+      if (family.org == "regr+" | family.org == "class+" | family.org == "mix+") {
+        cat("              Total no. of responses: ", yvar.dim,                 "\n", sep="")
+        cat("         User has requested response: ", outcome.target,           "\n", sep="")
+      }
       cat("           Variables sorted by VIMP?: ", sorted,                       "\n", sep="")
       cat("   No. of variables used for pairing: ", n.interact,                   "\n", sep="")
       cat("    Total no. of paired interactions: ", length(rownames.interact.imp),"\n", sep="")
@@ -191,24 +202,24 @@ find.interaction.rfsrc <- function(
     }
     invisible(interact.imp)
   }
-  else {
-    max.obj <- max.subtree(object, sub.order = TRUE, max.order = 1)
-    sub.order <- max.obj$sub.order
-    if (sorted) {
-      o.r <- order(diag(sub.order), decreasing = FALSE)
-      sub.order <- sub.order[o.r, o.r]
+    else {
+      max.obj <- max.subtree(object, sub.order = TRUE, max.order = 1)
+      sub.order <- max.obj$sub.order
+      if (sorted) {
+        o.r <- order(diag(sub.order), decreasing = FALSE)
+        sub.order <- sub.order[o.r, o.r]
+      }
+      xvar.pt <- is.element(colnames(sub.order), xvar.names[1:n.interact])
+      sub.order <- sub.order[xvar.pt, xvar.pt]
+      if (verbose) {
+        cat("\n")
+        cat("                              Method: ", method,              "\n", sep="")
+        cat("                    No. of variables: ", n.interact,          "\n", sep="")
+        cat("  Variables sorted by minimal depth?: ", sorted,              "\n", sep="")
+        cat("\n")
+        print(round(sub.order, 2))
+      }
+      invisible(sub.order)
     }
-    cov.pt <- is.element(colnames(sub.order), cov.names[1:n.interact])
-    sub.order <- sub.order[cov.pt, cov.pt]
-    if (verbose) {
-      cat("\n")
-      cat("                              Method: ", method,              "\n", sep="")
-      cat("                    No. of variables: ", n.cov,               "\n", sep="")
-      cat("  Variables sorted by minimal depth?: ", sorted,              "\n", sep="")
-      cat("\n")
-      print(round(sub.order, 2))
-    }
-    invisible(sub.order)
-  }
 }
 find.interaction <- find.interaction.rfsrc
