@@ -2,9 +2,9 @@
 ####**********************************************************************
 ####
 ####  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-####  Version 1.6.1
+####  Version 2.0.0 (_PROJECT_BUILD_ID_)
 ####
-####  Copyright 2012, University of Miami
+####  Copyright 2015, University of Miami
 ####
 ####  This program is free software; you can redistribute it and/or
 ####  modify it under the terms of the GNU General Public License
@@ -52,7 +52,7 @@
 ####    5425 Nestleway Drive, Suite L1
 ####    Clemmons, NC 27012
 ####
-####    email:  commerce@kogalur.com
+####    email:  ubk@kogalur.com
 ####    URL:    http://www.kogalur.com
 ####    --------------------------------------------------------------
 ####
@@ -60,7 +60,7 @@
 ####**********************************************************************
 
 
-print.rfsrc <- function(x, ...) {
+print.rfsrc <- function(x, outcome.target = NULL, ...) {
   if (sum(inherits(x, c("rfsrc", "forest"), TRUE) == c(1, 2)) == 2) {
     print.default(x)
     return()
@@ -69,7 +69,10 @@ print.rfsrc <- function(x, ...) {
     print.default(x)
     return()
   }
+  sf.flag <- FALSE
   if (sum(inherits(x, c("rfsrc", "synthetic"), TRUE) == c(1, 2)) == 2) {
+    sf.flag <- TRUE
+    sf.message <- "OOB performance values are unreliable for synthetic forests, consider using cross-validation"
     x <- x$rfSyn
   }
   if (sum(inherits(x, c("rfsrc", "grow"), TRUE) == c(1, 2)) != 2 &
@@ -79,9 +82,13 @@ print.rfsrc <- function(x, ...) {
   if (sum(inherits(x, c("rfsrc", "grow"), TRUE) == c(1, 2)) == 2) {
     grow.mode <- TRUE
   }
-  else {
-    grow.mode <- FALSE
-  }
+    else {
+      grow.mode <- FALSE
+    }
+  family.org <- x$family
+  yvar.dim <- ncol(x$yvar)
+  outcome.target <- coerce.multivariate.target(x, outcome.target)
+  x <- coerce.multivariate(x, outcome.target)
   if (grepl("surv", x$family)) {
     event <- get.event.info(x)$event
     n.event <- 1
@@ -90,50 +97,49 @@ print.rfsrc <- function(x, ...) {
       if (length(event) > 0) {
         event.freq <- paste(tapply(event, event, length), collapse = ", ")
       }
-      else {
-        event.freq <- 0
-      }
+        else {
+          event.freq <- 0
+        }
     }
   }
   if (x$family == "class") {
     if (!is.null(x$yvar)) {
       event.freq <- paste(tapply(x$yvar, x$yvar, length), collapse = ", ")
     }
-    else {
-      event.freq <- NA
-    }
+      else {
+        event.freq <- NA
+      }
     if (!is.null(x$err.rate)) {
       conf.matx <- table(x$yvar, if(!is.null(x$class.oob) && !all(is.na(x$class.oob))) x$class.oob else x$class)
       conf.matx <- cbind(conf.matx,  class.error = round(1 - diag(conf.matx)/rowSums(conf.matx, na.rm = TRUE), 4))
       names(dimnames(conf.matx)) <- c("  observed", "predicted")
-      brierS <- brier(x$yvar,
-                 if(!is.null(x$predicted.oob) && !all(is.na(x$predicted.oob))) x$predicted.oob else x$predicted)
-    }
-    else {
-      conf.matx <- brierS <- NULL
+      brierS <- brier(x$yvar, if(!is.null(x$predicted.oob) && !all(is.na(x$predicted.oob))) x$predicted.oob else x$predicted)
     }
+      else {
+        conf.matx <- brierS <- NULL
+      }
   }
   if (!is.null(x$err.rate)) {
     err.rate <- cbind(x$err.rate)
     if (grepl("surv", x$family)) {
       err.rate <- paste(round(100 * err.rate[nrow(err.rate), ], 2), "%", collapse=", ", sep = "")
     }
-    else if (x$family == "class") {
-      overall.err.rate <- paste(round(100 * err.rate[nrow(err.rate), 1], 2), "%", sep = "")
-      err.rate <- paste(round(err.rate[nrow(err.rate), ], 2), collapse=", ", sep = "")
-      brierS <- round(100 * brierS, 2)
-    }
-    else if (x$family == "regr") {
-      per.var <- round(100 * (1 - err.rate[nrow(err.rate), ] / var(x$yvar, na.rm = TRUE)), 2)
-      err.rate <- round(err.rate[nrow(err.rate), ], 2)
-    }
+      else if (x$family == "class") {
+        overall.err.rate <- paste(round(100 * err.rate[nrow(err.rate), 1], 2), "%", sep = "")
+        err.rate <- paste(round(err.rate[nrow(err.rate), ], 2), collapse=", ", sep = "")
+        brierS <- round(100 * brierS, 2)
+      }
+        else if (x$family == "regr") {
+          per.var <- round(100 * (1 - err.rate[nrow(err.rate), ] / var(x$yvar, na.rm = TRUE)), 2)
+          err.rate <- round(err.rate[nrow(err.rate), ], 2)
+        }
+          else {
+            err.rate <- NULL
+          }
+  }
     else {
       err.rate <- NULL
     }
-  }
-  else {
-    err.rate <- NULL
-  }
   if (is.null(x$nsplit)) {
     x$nsplit <- 0
   }
@@ -143,41 +149,42 @@ print.rfsrc <- function(x, ...) {
       if (n.event > 1) {
         cat("                    Number of events: ", event.freq,  "\n", sep="")
       }
-      else {
-        cat("                    Number of deaths: ", x$ndead,   "\n", sep="")
-      }
+        else {
+          cat("                    Number of deaths: ", x$ndead,   "\n", sep="")
+        }
     }
     if (x$family == "class") {
       cat("           Frequency of class labels: ", event.freq,          "\n", sep="")
     }
     if (!is.null(x$imputed.indv)) {
       cat("                    Was data imputed: ", "yes",               "\n", sep="")
-      #cat("                         Missingness: ",
-      #    round(100*length(x$imputed.indv)/x$n,2), "%\n", sep="")      
+                                        #cat("                         Missingness: ",
+                                        #    round(100*length(x$imputed.indv)/x$n,2), "%\n", sep="")      
     }
     cat("                     Number of trees: ", x$ntree,                "\n",sep="")
     cat("          Minimum terminal node size: ", x$nodesize,             "\n", sep="")
     cat("       Average no. of terminal nodes: ", mean(x$leaf.count),     "\n", sep="")
     cat("No. of variables tried at each split: ", x$mtry,                 "\n", sep="")
     cat("              Total no. of variables: ", length(x$xvar.names),   "\n", sep="")
-    if (x$family == "regr+" | x$family == "class+" | x$family == "mix+") { 
-      cat("              Total no. of responses: ", length(x$yvar.names),   "\n", sep="")
+    if (!x$univariate) { 
+      cat("              Total no. of responses: ", yvar.dim,   "\n", sep="")
+      cat("         User has requested response: ", outcome.target,        "\n", sep="")
     }
-    cat("                            Analysis: ", family.pretty(x$family),"\n", sep="")
-    cat("                              Family: ", x$family,              "\n", sep="")
+    cat("                            Analysis: ", family.pretty(family.org),"\n", sep="")
+    cat("                              Family: ", family.org,               "\n", sep="")
     if (x$nsplit > 0 & x$splitrule != "random") {
       cat("                      Splitting rule: ", paste(x$splitrule,"*random*"),"\n", sep="")
       cat("       Number of random split points: ", x$nsplit                   ,  "\n", sep="")
     }
-    else {
-      cat("                      Splitting rule: ", x$splitrule,         "\n", sep="")
-    } 
+      else {
+        cat("                      Splitting rule: ", x$splitrule,         "\n", sep="")
+      } 
     if (!is.null(err.rate)) {
       if (x$family == "regr") {
         cat("                % variance explained: ", per.var, "\n", sep="")
       }
       if (x$family == "class" && !is.null(brierS)) {
-        cat("                         Brier score:", brierS, "\n")
+        cat("              Normalized Brier score:", brierS, "\n")
       } 
       cat("                          Error rate: ", err.rate,            "\n\n", sep="")
     }
@@ -185,54 +192,58 @@ print.rfsrc <- function(x, ...) {
       if (!is.null(x$predicted.oob) && any(is.na(x$predicted.oob))) {
         cat("Confusion matrix (cases with missing OOB predicted values have been removed):\n\n")
       }
-      else {
-        cat("Confusion matrix:\n\n")
-      }
+        else {
+          cat("Confusion matrix:\n\n")
+        }
       print(conf.matx)
       cat("\n\tOverall error rate:", overall.err.rate, "\n")
     }
   }
-  else {
-    cat("  Sample size of test (predict) data: ", x$n,  "\n", sep="")
-    if (grepl(x$family, "surv") && !is.null(event)) {
-      if (n.event > 1) {
-        cat("       Number of events in test data: ", event.freq,  "\n", sep="")
+    else {
+      cat("  Sample size of test (predict) data: ", x$n,  "\n", sep="")
+      if (grepl(x$family, "surv") && !is.null(event)) {
+        if (n.event > 1) {
+          cat("       Number of events in test data: ", event.freq,  "\n", sep="")
+        }
+          else {
+            cat("       Number of deaths in test data: ", unlist(event.freq),   "\n", sep="")
+          }
       }
-      else {
-        cat("       Number of deaths in test data: ", unlist(event.freq),   "\n", sep="")
+      if (!is.null(x$imputed.data)) {
+        cat("               Was test data imputed: ", "yes",               "\n", sep="")
+                                        #cat("                         Missingness: ",
+                                        #    round(100*length(x$imputed.indv)/x$n,2), "%\n", sep="")      
+      }
+      cat("                Number of grow trees: ", x$ntree,             "\n",sep="")
+      cat("  Average no. of grow terminal nodes: ", mean(x$leaf.count),  "\n", sep="")
+      cat("         Total no. of grow variables: ", length(x$xvar.names), "\n", sep="")  
+      if (!x$univariate) { 
+        cat("         Total no. of grow responses: ", yvar.dim,   "\n", sep="")
+        cat("         User has requested response: ", outcome.target,        "\n", sep="")
+      }
+      cat("                            Analysis: ", family.pretty(family.org),"\n", sep="")
+      cat("                              Family: ", family.org,               "\n", sep="")
+      if (!is.null(err.rate)) {
+        if (x$family == "regr") {
+          cat("                % variance explained: ", per.var, "\n", sep="")
+        }
+        if (x$family == "class" && !is.null(brierS)) {
+          cat("       Test set Normalized Brier score:", brierS, "\n")
+        }
+        cat("                 Test set error rate: ", err.rate, "\n\n", sep="")
+      }
+      if (x$family == "class" && !is.null(conf.matx)) {
+        if (!is.null(x$predicted.oob) && any(is.na(x$predicted.oob))) {
+          cat("Confusion matrix (cases with missing OOB predicted values have been removed):\n\n")
+        }
+          else {
+            cat("Confusion matrix:\n\n")
+          }
+        print(conf.matx)
+        cat("\n\tOverall error rate:", overall.err.rate, "\n")
       }
     }
-    if (!is.null(x$imputed.data)) {
-      cat("               Was test data imputed: ", "yes",               "\n", sep="")
-      #cat("                         Missingness: ",
-      #    round(100*length(x$imputed.indv)/x$n,2), "%\n", sep="")      
-    }
-    cat("                Number of grow trees: ", x$ntree,             "\n",sep="")
-    cat("  Average no. of grow terminal nodes: ", mean(x$leaf.count),  "\n", sep="")
-    cat("         Total no. of grow variables: ", length(x$xvar.names), "\n", sep="")  
-    if (x$family == "regr+" | x$family == "class+" | x$family == "mix+") { 
-      cat("         Total no. of grow responses: ", length(x$yvar.names),   "\n", sep="")
-    }
-    cat("                            Analysis: ", family.pretty(x$family),"\n", sep="")
-    cat("                              Family: ", x$family,              "\n", sep="")
-    if (!is.null(err.rate)) {
-      if (x$family == "regr") {
-        cat("                % variance explained: ", per.var, "\n", sep="")
-      }
-      if (x$family == "class" && !is.null(brierS)) {
-      cat("                Test set Brier score:", brierS, "\n")
-      }
-      cat("                 Test set error rate: ", err.rate, "\n\n", sep="")
-    }
-    if (x$family == "class" && !is.null(conf.matx)) {
-     if (!is.null(x$predicted.oob) && any(is.na(x$predicted.oob))) {
-        cat("Confusion matrix (cases with missing OOB predicted values have been removed):\n\n")
-      }
-      else {
-        cat("Confusion matrix:\n\n")
-      }
-       print(conf.matx)
-      cat("\n\tOverall error rate:", overall.err.rate, "\n")
-    }
+  if (sf.flag) {
+    message(sf.message)
   }
 }
